@@ -5,18 +5,23 @@
  *
  *  Copyright 2002 Iain Holmes
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of version 2 of the GNU General Public
- *  License as published by the Free Foundation.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * 4th Februrary 2005: Christian Schaller: changed license to LGPL with
+ * permission of Iain Holmes, Ronald Bultje, Leontine Binchy (SUN), Johan Dalhin * and Joe Marcus Clarke
  *
  */
 
@@ -29,7 +34,7 @@
 #include <unistd.h>
 
 #include <gnome.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <gconf/gconf-client.h>
 #include <gst/gst.h>
 #include <gst/gconf/gconf.h>
@@ -53,6 +58,7 @@ enum {
 #define KEY_SAVE_DIR        GCONF_DIR "system-state/save-file-directory"
 #define KEY_KVPS            GCONF_DIR "UIConf/kvps"
 #define KEY_LAST_PROFILE_ID GCONF_DIR "last-profile-id"
+#define EBUSY_TRY_AGAIN     3000    /* Empirical data */
 
 #define CMD_SET_SENSITIVE(window, key, value)               \
   bonobo_ui_component_set_prop (window->priv->ui_component, \
@@ -442,6 +448,41 @@ cb_iterate (GstBin  *bin,
 }
 #endif
 
+static gboolean
+is_set_ebusy_timeout (GSRWindow *window)
+{
+	if (g_object_get_data (G_OBJECT (window), "ebusy_timeout"))
+		return TRUE;
+
+	return FALSE;
+}
+
+static void
+set_ebusy_timeout (GSRWindow *window, gpointer data)
+{
+	g_object_set_data (G_OBJECT (window), "ebusy_timeout", data);
+}
+
+static gboolean
+handle_ebusy_error (GSRWindow *window)
+{
+	GSRWindowPrivate *priv = window->priv;
+
+	/* FIXME: which pipeline to reset state on? */
+	if (priv->play) {
+		gst_element_set_state (priv->play->pipeline, GST_STATE_NULL);
+		gst_element_set_state (priv->play->pipeline, GST_STATE_PLAYING);
+	} else if (priv->record) {
+		gst_element_set_state (priv->record->pipeline, GST_STATE_NULL);
+		gst_element_set_state (priv->record->pipeline, GST_STATE_PLAYING);
+	} else {
+		g_warning ("Don't know which pipeline to reset");
+	}
+
+	/* Try only once */
+	return FALSE;
+}
+
 static void
 pipeline_error_cb (GstElement *parent,
 		   GstElement *cause,
@@ -455,6 +496,21 @@ pipeline_error_cb (GstElement *parent,
 
 	g_return_if_fail (GTK_IS_WINDOW (window));
 	
+	if (error->code == GST_RESOURCE_ERROR_BUSY) {
+		if (! is_set_ebusy_timeout (window)) {
+			g_timeout_add (EBUSY_TRY_AGAIN, (GSourceFunc) handle_ebusy_error, window);
+			set_ebusy_timeout (window, GUINT_TO_POINTER (TRUE));
+
+			CMD_SET_SENSITIVE (window, "FileSave", "0");
+			CMD_SET_SENSITIVE (window, "FileSaveAs", "0");
+			CMD_SET_SENSITIVE (window, "MediaPlay", "0");
+			CMD_SET_SENSITIVE (window, "MediaRecord", "0");
+			return;
+		}
+	}
+
+	set_ebusy_timeout (window, NULL);
+
 	show_error_dialog (GTK_WINDOW (window), error->message);
 
 	ed = g_new (struct _eos_data, 1);
@@ -470,45 +526,183 @@ pipeline_error_cb (GstElement *parent,
 	eos_done (ed);
 }
 
+GtkWidget *
+gsr_button_new_with_stock_image (const gchar *text, const gchar *stock_id)
+{
+	GtkWidget *button;
+	GtkStockItem item;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *hbox;
+	GtkWidget *align;
+
+	button = gtk_button_new ();
+
+	if (GTK_BIN (button)->child)
+		gtk_container_remove (GTK_CONTAINER (button),
+				      GTK_BIN (button)->child);
+
+	if (gtk_stock_lookup (stock_id, &item)) {
+		label = gtk_label_new_with_mnemonic (text);
+
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+		hbox = gtk_hbox_new (FALSE, 2);
+
+		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+		gtk_container_add (GTK_CONTAINER (button), align);
+		gtk_container_add (GTK_CONTAINER (align), hbox);
+
+		gtk_widget_show_all (align);
+
+		return button;
+
+	}
+
+	label = gtk_label_new_with_mnemonic (text);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+	gtk_widget_show (label);
+	gtk_container_add (GTK_CONTAINER (button), label);
+
+	return button;
+}
+
+GtkWidget *
+gsr_dialog_add_button (GtkDialog *dialog, const gchar *text, const gchar *stock_id, gint response_id)
+{
+	GtkWidget *button;
+
+	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
+	g_return_val_if_fail (text != NULL, NULL);
+	g_return_val_if_fail (stock_id != NULL, NULL);
+
+	button = gsr_button_new_with_stock_image (text, stock_id);
+	g_return_val_if_fail (button != NULL, NULL);
+
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+	gtk_widget_show (button);
+
+	gtk_dialog_add_action_widget (dialog, button, response_id);
+
+	return button;
+}
+
+static gboolean
+replace_dialog (GtkWindow *parent,
+		const gchar *message,
+		const gchar *file_name)
+{
+	GtkWindow *message_dialog;
+	gint ret;
+
+	g_return_val_if_fail (file_name != NULL, FALSE);
+
+	message_dialog = gtk_message_dialog_new (parent,
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_QUESTION,
+						 GTK_BUTTONS_NONE,
+						 message,
+						 file_name);
+	/* Add cancel button */
+	gtk_dialog_add_button (GTK_DIALOG (message_dialog),
+			       GTK_STOCK_CANCEL,
+			       GTK_RESPONSE_CANCEL);
+	/* Add replace button */
+	gsr_dialog_add_button (GTK_DIALOG (message_dialog), _("_Replace"),
+			       GTK_STOCK_REFRESH,
+			       GTK_RESPONSE_YES);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (message_dialog), GTK_RESPONSE_CANCEL);
+	gtk_window_set_resizable (GTK_WINDOW (message_dialog), FALSE);
+	ret = gtk_dialog_run (GTK_DIALOG (message_dialog));
+	gtk_widget_destroy (GTK_WIDGET (message_dialog));
+
+	return (ret == GTK_RESPONSE_YES);
+}
+
+static gboolean
+replace_existing_file (GtkWindow *parent,
+		      const gchar *file_name) 
+{
+	return replace_dialog (parent,
+			       _("A file named \"%s\" already exists. \n"
+			       "Do you want to replace it with the "
+			       "one you are saving?"),
+			       file_name);
+}
+
 static void
 do_save_file (GSRWindow *window,
-	      const char *name)
+	      const char *_name)
 {
 	GSRWindowPrivate *priv;
         GMAudioProfile *profile;
-	char *tmp, *command;
-	GError *error;
-	int status;
-	
-	status = 0;
+	char *tmp, *src, *name;
+	GnomeVFSURI *src_uri, *dst_uri;
 
 	priv = window->priv;
-	
+
         profile = gm_audio_profile_choose_get_active (priv->profile);
 
-	tmp = g_strdup_printf ("%s.%s", name,
+	if (g_str_has_suffix (_name, gm_audio_profile_get_extension (profile)))
+		name = g_strdup (_name);
+	else
+		name = g_strdup_printf ("%s.%s", _name,
 			       gm_audio_profile_get_extension (profile));
-	
-	command = g_strdup_printf ("/bin/cp %s %s", priv->record_filename, tmp);
-
-	if (!g_spawn_command_line_sync (command,  NULL, NULL, &status, &error)) {
-		gchar *error_message;
-
-		error_message = g_strdup_printf (_("Could not save the file \" %s\""), error->message);
-		show_error_dialog (GTK_WINDOW (window), error_message);
-		g_free (error_message);
+	if (g_file_test (name, G_FILE_TEST_EXISTS)) {
+		if (!replace_existing_file (GTK_WINDOW (window), name))
+			return;
 	}
-	
-	if (status) {
+
+	tmp = g_strdup_printf ("file://%s", name);
+	src = g_strdup_printf ("file://%s", priv->record_filename);
+	src_uri = gnome_vfs_uri_new (src);
+	dst_uri = gnome_vfs_uri_new (tmp);
+	g_free (src);
+	if (src_uri && dst_uri) {
+		GnomeVFSResult result;
+		result = gnome_vfs_xfer_uri (src_uri, dst_uri,
+			GNOME_VFS_XFER_DEFAULT,
+			GNOME_VFS_XFER_ERROR_MODE_ABORT,
+			GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
+			NULL, NULL);
+		if (result == GNOME_VFS_OK) {
+			char *title, *short_name;
+			priv->filename = g_strdup (name);
+			short_name = g_path_get_basename (priv->filename);
+			title = g_strdup_printf ("%s - Sound Recorder",
+				short_name);
+			gtk_window_set_title (GTK_WINDOW (window), title);
+			gtk_label_set (GTK_LABEL (priv->name), short_name);
+			priv->dirty = FALSE;
+			g_free (title);
+		} else {
+			gchar *error_message;
+ 
+			error_message = g_strdup_printf (_("Could not save the file \" %s\""), gnome_vfs_result_to_string (result));
+			show_error_dialog (GTK_WINDOW (window), error_message);
+			g_free (error_message);
+		}
+		gnome_vfs_uri_unref (src_uri);
+		gnome_vfs_uri_unref (dst_uri);
+	} else {
 		gchar *error_message;
 
-		error_message = g_strdup_printf (_("Could not save the file \" %s\""), tmp);
+		error_message = g_strdup_printf (_("Could not save the file \"%s\""), name);
 		show_error_dialog (GTK_WINDOW (window), error_message);
 		g_free (error_message);
 	}
 
-	g_free (tmp);		
-	g_free (command);		
+	g_free (name);
+	g_free (tmp);
 }
 
 static void
@@ -567,10 +761,28 @@ file_save_as (BonoboUIComponent *uic,
 
 	if (window->priv->filename != NULL) {
 		char *basename;
+		gchar *filename, *filename_ext, *extension;
+		gint length;
+		GMAudioProfile *profile;
 
 		basename = g_path_get_basename (window->priv->filename);
+		length = strlen (basename);
+		extension = strrchr (basename, '.');
+
+		if (extension != NULL) {
+			length = length - strlen (extension);
+		}
+
+		filename = g_strndup (basename,length);
+		profile = gm_audio_profile_choose_get_active (window->priv->profile);
+		g_free (window->priv->extension);
+
+		window->priv->extension = g_strdup (gm_audio_profile_get_extension (profile));
+		filename_ext = g_strdup_printf ("%s.%s", filename , window->priv->extension);
 		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_chooser),
-						   basename);
+						   filename_ext);
+		g_free (filename);
+		g_free (filename_ext);
 		g_free (basename);
 	}
 
@@ -585,7 +797,12 @@ file_save (BonoboUIComponent *uic,
 	   const char *path)
 {
 	if (window->priv->filename == NULL ||
-	    strncmp (window->priv->filename, "Untitled", 8) == 0) {
+	    /* Translator comment: Untitled here implies a track without a
+	     * name. See also the translation in gnome-recorder.c:94. Those
+	     * two strings should match! If the track is unnamed, we will
+	     * open the save-as dialog here, else we´ll use the given file
+	     * to save to. */
+	    g_strrstr (window->priv->filename, _("Untitled")) == 0) {
 		file_save_as (uic, window, NULL);
 	} else {
 		do_save_file (window, window->priv->filename);
@@ -706,7 +923,8 @@ fill_in_information (GSRWindow *window,
 		char *human;
 		human = gnome_vfs_format_file_size_for_display (buf.st_size);
 
-		text = g_strdup_printf ("%s (%llu bytes)", human, buf.st_size);
+		text = g_strdup_printf (ngettext ("%s (%llu byte)", "%s (%llu bytes)", 
+						   buf.st_size), human, buf.st_size);
 		g_free (human);
 	} else {
 		text = g_strdup (_("Unknown size"));
@@ -853,7 +1071,7 @@ file_properties (BonoboUIComponent *uic,
 	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 	gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
 
-	label = make_info_label (_("Song length:"));
+	label = make_info_label (_("File duration:"));
 	pack_table_widget (table, label, 0, 0);
 
 	fp->length = make_info_label ("");
@@ -1014,13 +1232,12 @@ media_record (BonoboUIComponent *uic,
 		g_object_unref (priv->record->pipeline);
 	}
 	priv->record = make_record_pipeline (window);
+	if (!priv->record)
+		return;
 
-	if (priv->record != NULL) {
-		g_object_set (G_OBJECT (priv->record->sink),
-			      "location", priv->record_filename,
-			      NULL);
-	}
-	
+	g_object_set (G_OBJECT (priv->record->sink),
+		      "location", priv->record_filename,
+		      NULL);
 	window->priv->len_secs = 0;
 	gst_element_set_state (priv->record->pipeline, GST_STATE_PLAYING);
 }
@@ -1210,6 +1427,7 @@ play_state_changed_cb (GstElement *element,
 		bonobo_ui_component_set_status (window->priv->ui_component,
 						_("Playing..."), NULL);
 		gtk_widget_set_sensitive (window->priv->scale, TRUE);
+		set_ebusy_timeout (window, NULL);
 		break;
 
 	case GST_STATE_READY:
@@ -1253,29 +1471,6 @@ pipeline_deep_notify_cb (GstElement *element,
 	}
 }
 
-/* helper function to change the UI when file extension changes */
-static void
-set_extension (GSRWindow *window)
-{
-	gchar *short_name, *filename;
-        GMAudioProfile *profile;
-        g_object_get (window, "location", &filename, NULL);
-	short_name = g_path_get_basename (filename);
-        profile = gm_audio_profile_choose_get_active (window->priv->profile);
-        g_free (window->priv->extension);
-        window->priv->extension = g_strdup (gm_audio_profile_get_extension (profile));
-	if (filename != NULL) {
-		char *title;
-
-		title = g_strdup_printf (_("%s.%s - Sound Recorder"), short_name, window->priv->extension);
-		gtk_window_set_title (GTK_WINDOW (window), title);
-		g_free (title);
-	} else {
-		gtk_window_set_title (GTK_WINDOW (window), _("Sound Recorder"));
-	}
-
-}
-
 /* callback for when the recording profile has been changed */
 static void
 profile_changed_cb (GObject *object, GSRWindow *window)
@@ -1292,8 +1487,6 @@ profile_changed_cb (GObject *object, GSRWindow *window)
   gconf_client_set_string (client, KEY_LAST_PROFILE_ID, id, NULL);
   g_object_unref (G_OBJECT (client));
   g_free (id);
-
-  set_extension (window);
 }
 
 static GSRWindowPipeline *
@@ -1302,7 +1495,7 @@ make_play_pipeline (GSRWindow *window)
 	GSRWindowPipeline *obj;
 	GstElement *pipeline;
 	guint32 id;
-	GstElement *spider;
+	GstElement *spider, *conv, *scale;
 
 	pipeline = gst_pipeline_new ("play-pipeline");
 	g_signal_connect (pipeline, "deep-notify",
@@ -1328,16 +1521,19 @@ make_play_pipeline (GSRWindow *window)
 		g_error ("Could not find element spider");
 		return NULL;
 	}
-	
+
+	conv = gst_element_factory_make ("audioconvert", "conf");
+	scale = gst_element_factory_make ("audioscale", "scale");
 	obj->sink = gst_gconf_get_default_audio_sink ();
 	if (!obj->sink)
 	{
-		g_error ("Could not find default audio src element");
+		g_error ("Could not find default audio sink element");
 		return NULL;
 	}
 
-	gst_bin_add_many (GST_BIN (pipeline), obj->src, spider, obj->sink, NULL);
-	gst_element_link_many (obj->src, spider, obj->sink, NULL);
+	gst_bin_add_many (GST_BIN (pipeline),
+			  obj->src, spider, conv, scale, obj->sink, NULL);
+	gst_element_link_many (obj->src, spider, conv, scale, obj->sink, NULL);
 
 	return obj;
 }
@@ -1618,8 +1814,6 @@ gsr_window_new (const char *filename)
 			  1, 2, 1, 2,
 			  GTK_FILL, GTK_FILL,
 			  0, 0);
-			       
-        set_extension (window);
 
 	gtk_widget_show_all (window->priv->main_vbox);
 	return GTK_WIDGET (window);
@@ -1690,7 +1884,11 @@ gsr_window_class_init (GSRWindowClass *klass)
 					 g_param_spec_string ("location",
 							      "Location",
 							      "",
-							      "Untitled",
+	/* Translator comment: default trackname is 'untitled', which
+	 * has as effect that the user cannot save to this file. The
+	 * 'save' action will open the save-as dialog instead to give
+	 * a proper filename. See gnome-record.c:94. */
+							      _("Untitled"),
 							      G_PARAM_READWRITE));
 }
 
